@@ -1,36 +1,37 @@
-from django.contrib.auth.models import User
-from django.core.management import call_command
-from django.core.urlresolvers import reverse
-from django.test import TestCase
-try:
-    from unittest import skipIf
-except ImportError:
-    from django.utils.unittest import skipIf
-from django.test.client import Client
-try:
-    from django.test import override_settings
-except ImportError:
-    from django.test.utils import override_settings
+import datetime
+import time
+from unittest import skipIf, mock
+from unittest.mock import patch, PropertyMock, MagicMock
+from uuid import uuid4
+
 from django.conf import settings
+from django.core.management import call_command
+from django.test import TestCase, override_settings
+from django.urls import reverse
 
 from rq import get_current_job, Queue
+from rq.exceptions import NoSuchJobError
 from rq.job import Job
-from rq.registry import (DeferredJobRegistry, FinishedJobRegistry,
-                         StartedJobRegistry)
+from rq.registry import FinishedJobRegistry, ScheduledJobRegistry
+from rq.worker import Worker
 
 from django_rq.decorators import job
+from django_rq.jobs import get_job_class
 from django_rq.queues import (
-    get_connection, get_queue, get_queue_by_index, get_queues,
-    get_unique_connection_configs, DjangoRQ
+    get_connection, get_queue, get_queues,
+    get_unique_connection_configs, DjangoRQ,
+    get_redis_connection
 )
 from django_rq import thread_queue
 from django_rq.templatetags.django_rq import to_localtime
-from django_rq.workers import get_worker
-
+from django_rq.tests.fixtures import DummyJob, DummyQueue, DummyWorker
+from django_rq.utils import get_jobs, get_statistics
+from django_rq.workers import get_worker, get_worker_class
 
 try:
     from rq_scheduler import Scheduler
     from ..queues import get_scheduler
+    from django_rq.tests.fixtures import DummyScheduler
     RQ_SCHEDULER_INSTALLED = True
 except ImportError:
     RQ_SCHEDULER_INSTALLED = False
@@ -46,36 +47,43 @@ def divide(a, b):
     return a / b
 
 
-def get_failed_queue_index(name='default'):
-    """
-    Returns the position of FailedQueue for the named queue in QUEUES_LIST
-    """
-    # Get the index of FailedQueue for 'default' Queue in QUEUES_LIST
-    queue_index = None
-    connection = get_connection(name)
-    connection_kwargs = connection.connection_pool.connection_kwargs
-    for i in range(0, 100):
-        q = get_queue_by_index(i)
-        if q.name == 'failed' and q.connection.connection_pool.connection_kwargs == connection_kwargs:
-            queue_index = i
-            break
-
-    return queue_index
+def long_running_job(timeout=10):
+    time.sleep(timeout)
+    return 'Done sleeping...'
 
 
-def get_queue_index(name='default'):
-    """
-    Returns the position of Queue for the named queue in QUEUES_LIST
-    """
-    queue_index = None
-    connection = get_connection(name)
-    connection_kwargs = connection.connection_pool.connection_kwargs
-    for i in range(0, 100):
-        q = get_queue_by_index(i)
-        if q.name == name and q.connection.connection_pool.connection_kwargs == connection_kwargs:
-            queue_index = i
-            break
-    return queue_index
+def flush_registry(registry):
+    connection = registry.connection
+    for job_id in registry.get_job_ids():
+        connection.zrem(registry.key, job_id)
+        try:
+            job = Job.fetch(job_id, connection=connection)
+            job.delete()
+        except NoSuchJobError:
+            pass
+
+
+class RqStatsTest(TestCase):
+
+    def test_get_connection_default(self):
+        """
+        Test that rqstats returns the right statistics
+        """
+        # Override testing RQ_QUEUES
+        queues = [{
+            'connection_config': {
+                'DB': 0,
+                'HOST': 'localhost',
+                'PORT': 6379,
+            },
+            'name': 'default'
+        }]
+        with patch('django_rq.utils.QUEUES_LIST',
+                   new_callable=PropertyMock(return_value=queues)):
+            # Only to make sure it doesn't crash
+            call_command('rqstats')
+            call_command('rqstats', '-j')
+            call_command('rqstats', '-y')
 
 
 @override_settings(RQ={'AUTOCOMMIT': True})
@@ -105,6 +113,51 @@ class QueuesTest(TestCase):
         self.assertEqual(connection_kwargs['port'], config['PORT'])
         self.assertEqual(connection_kwargs['db'], config['DB'])
 
+    @patch('django_rq.queues.Sentinel')
+    def test_get_connection_sentinel(self, sentinel_class_mock):
+        """
+        Test that get_connection returns the right connection based for
+        `sentinel` queue.
+        """
+        sentinel_mock = MagicMock()
+        sentinel_mock.master_for.return_value = sentinel_mock
+        sentinel_class_mock.side_effect = [sentinel_mock]
+
+        config = QUEUES['sentinel']
+        connection = get_connection('sentinel')
+
+        self.assertEqual(connection, sentinel_mock)
+        self.assertEqual(sentinel_mock.master_for.call_count, 1)
+        self.assertEqual(sentinel_class_mock.call_count, 1)
+
+        sentinel_instances = sentinel_class_mock.call_args[0][0]
+        self.assertListEqual(config['SENTINELS'], sentinel_instances)
+
+        connection_kwargs = sentinel_mock.master_for.call_args[1]
+        self.assertEqual(connection_kwargs['service_name'],
+                         config['MASTER_NAME'])
+
+    @patch('django_rq.queues.Sentinel')
+    def test_sentinel_class_initialized_with_kw_args(self, sentinel_class_mock):
+        """
+        Test that Sentinel object is initialized with proper connection kwargs.
+        """
+        config = {
+            'SENTINELS': [],
+            'MASTER_NAME': 'test_master',
+            'SOCKET_TIMEOUT': 0.2,
+            'DB': 0,
+            'CONNECTION_KWARGS': {
+                'socket_connect_timeout': 0.3
+            }
+        }
+        get_redis_connection(config)
+        sentinel_init_kwargs = sentinel_class_mock.call_args[1]
+        self.assertDictEqual(
+            sentinel_init_kwargs,
+            {'socket_connect_timeout': 0.3, 'db': 0,
+             'socket_timeout': 0.2, 'password': None})
+
     def test_get_queue_default(self):
         """
         Test that get_queue use the right parameters for `default`
@@ -153,7 +206,6 @@ class QueuesTest(TestCase):
         connection, where no DB given and URL does not contain the db number
         (redis-py defaults to 0, should not break).
         """
-        config = QUEUES['url_default_db']
         queue = get_queue('url_default_db')
         connection_kwargs = queue.connection.connection_pool.connection_kwargs
         self.assertEqual(queue.name, 'url_default_db')
@@ -188,6 +240,13 @@ class QueuesTest(TestCase):
         """
         self.assertRaises(ValueError, get_queues, 'default', 'test')
 
+    def test_get_queues_different_classes(self):
+        """
+        Checks that getting queues with different classes (defined in configuration)
+        raises an exception.
+        """
+        self.assertRaises(ValueError, get_queues, 'test', 'test1')
+
     def test_pass_queue_via_commandline_args(self):
         """
         Checks that passing queues via commandline arguments works
@@ -207,6 +266,31 @@ class QueuesTest(TestCase):
             self.assertTrue(job['job'].is_finished)
             self.assertIn(job['job'].id, job['finished_job_registry'].get_job_ids())
 
+    @mock.patch('rq.contrib.sentry.register_sentry')
+    def test_sentry_dsn(self, mocked):
+        queue_names = ['django_rq_test']
+        call_command('rqworker', *queue_names, burst=True,
+                     sentry_dsn='https://1@sentry.io/1')
+
+        self.assertEqual(mocked.call_count, 1)
+
+    @mock.patch('rq.contrib.sentry.register_sentry')
+    def test_sentry_dsn_setting(self, mocked):
+        queue_names = ['django_rq_test']
+        with self.settings(SENTRY_DSN='https://1@sentry.io/1'):
+            call_command('rqworker', *queue_names, burst=True)
+
+            self.assertEqual(mocked.call_count, 1)
+
+    @mock.patch('rq.contrib.sentry.register_sentry')
+    def test_sentry_dsn_setting_override(self, mocked):
+        queue_names = ['django_rq_test']
+        with self.settings(SENTRY_DSN='https://1@sentry.io/1'):
+            call_command('rqworker', *queue_names, burst=True,
+                         sentry_dsn='')
+
+            self.assertEqual(mocked.call_count, 0)
+
     def test_get_unique_connection_configs(self):
         connection_params_1 = {
             'HOST': 'localhost',
@@ -266,17 +350,24 @@ class QueuesTest(TestCase):
         """
         Checks whether asynchronous settings work
         """
-        # Make sure async is not set by default
+        # Make sure is_async is not set by default
         default_queue = get_queue('default')
-        self.assertTrue(default_queue._async)
+        self.assertTrue(default_queue._is_async)
 
-        # Make sure async override works
-        default_queue_async = get_queue('default', async=False)
-        self.assertFalse(default_queue_async._async)
+        # Make sure is_async override works
+        default_queue_is_async = get_queue('default', is_async=False)
+        self.assertFalse(default_queue_is_async._is_async)
 
-        # Make sure async setting works
+        # Make sure old keyword argument 'async' works for backwards
+        # compatibility with code expecting older versions of rq or django-rq.
+        # Note 'async' is a reserved keyword in Python >= 3.7.
+        kwargs = {'async': False}
+        default_queue_async = get_queue('default', **kwargs)
+        self.assertFalse(default_queue_async._is_async)
+
+        # Make sure is_async setting works
         async_queue = get_queue('async')
-        self.assertFalse(async_queue._async)
+        self.assertFalse(async_queue._is_async)
 
     @override_settings(RQ={'AUTOCOMMIT': False})
     def test_autocommit(self):
@@ -312,6 +403,7 @@ class DecoratorTest(TestCase):
         # Ensure that decorator passes in the right queue from settings.py
         queue_name = 'test3'
         config = QUEUES[queue_name]
+
         @job(queue_name)
         def test():
             pass
@@ -329,6 +421,25 @@ class DecoratorTest(TestCase):
         self.assertEqual(result.origin, 'default')
         result.delete()
 
+    def test_job_decorator_result_ttl_default(self):
+        from rq.defaults import DEFAULT_RESULT_TTL
+
+        @job
+        def test():
+            pass
+        result = test.delay()
+        self.assertEqual(result.result_ttl, DEFAULT_RESULT_TTL)
+        result.delete()
+
+    @override_settings(RQ={'AUTOCOMMIT': True, 'DEFAULT_RESULT_TTL': 5432})
+    def test_job_decorator_result_ttl(self):
+        @job
+        def test():
+            pass
+        result = test.delay()
+        self.assertEqual(result.result_ttl, 5432)
+        result.delete()
+
 
 @override_settings(RQ={'AUTOCOMMIT': True})
 class WorkersTest(TestCase):
@@ -350,6 +461,15 @@ class WorkersTest(TestCase):
         queue = w.queues[0]
         self.assertEqual(queue.name, 'test')
 
+    def test_get_worker_custom_classes(self):
+        w = get_worker('test',
+                       job_class='django_rq.tests.fixtures.DummyJob',
+                       queue_class='django_rq.tests.fixtures.DummyQueue',
+                       worker_class='django_rq.tests.fixtures.DummyWorker')
+        self.assertIs(w.job_class, DummyJob)
+        self.assertIsInstance(w.queues[0], DummyQueue)
+        self.assertIsInstance(w, DummyWorker)
+
     def test_get_current_job(self):
         """
         Ensure that functions using RQ's ``get_current_job`` doesn't fail
@@ -362,149 +482,18 @@ class WorkersTest(TestCase):
         self.assertFalse(job.id in failed_queue.job_ids)
         job.delete()
 
-
-@override_settings(RQ={'AUTOCOMMIT': True})
-class ViewTest(TestCase):
-
-    def setUp(self):
-        user = User.objects.create_user('foo', password='pass')
-        user.is_staff = True
-        user.is_active = True
-        user.save()
-        self.client = Client()
-        self.client.login(username=user.username, password='pass')
-        get_queue('django_rq_test').connection.flushall()
-
-    def test_requeue_job(self):
-        """
-        Ensure that a failed job gets requeued when rq_requeue_job is called
-        """
-        def failing_job():
-            raise ValueError
-
-        queue = get_queue('default')
-        queue_index = get_failed_queue_index('default')
-        job = queue.enqueue(failing_job)
-        worker = get_worker('default')
-        worker.work(burst=True)
-        job.refresh()
-        self.assertTrue(job.is_failed)
-        self.client.post(reverse('rq_requeue_job', args=[queue_index, job.id]),
-                         {'requeue': 'Requeue'})
-        self.assertIn(job, queue.jobs)
-        job.delete()
-
-    def test_delete_job(self):
-        """
-        In addition to deleting job from Redis, the job id also needs to be
-        deleted from Queue.
-        """
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-        job = queue.enqueue(access_self)
-        self.client.post(reverse('rq_delete_job', args=[queue_index, job.id]),
-                         {'post': 'yes'})
-        self.assertFalse(Job.exists(job.id, connection=queue.connection))
-        self.assertNotIn(job.id, queue.job_ids)
-
-    def test_action_delete_jobs(self):
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-
-        # enqueue some jobs
-        job_ids = []
-        for _ in range(0, 3):
-            job = queue.enqueue(access_self)
-            job_ids.append(job.id)
-
-        # remove those jobs using view
-        self.client.post(reverse('rq_actions', args=[queue_index]),
-                         {'action': 'delete', 'job_ids': job_ids})
-
-        # check if jobs are removed
-        for job_id in job_ids:
-            self.assertFalse(Job.exists(job_id, connection=queue.connection))
-            self.assertNotIn(job_id, queue.job_ids)
-
-    def test_action_requeue_jobs(self):
-        def failing_job():
-            raise ValueError
-
-        queue = get_queue('django_rq_test')
-        failed_queue_index = get_failed_queue_index('django_rq_test')
-
-        # enqueue some jobs that will fail
-        jobs = []
-        job_ids = []
-        for _ in range(0, 3):
-            job = queue.enqueue(failing_job)
-            jobs.append(job)
-            job_ids.append(job.id)
-
-        # do those jobs = fail them
-        worker = get_worker('django_rq_test')
-        worker.work(burst=True)
-
-        # check if all jobs are really failed
-        for job in jobs:
-            self.assertTrue(job.is_failed)
-
-        # renqueue failed jobs from failed queue
-        self.client.post(reverse('rq_actions', args=[failed_queue_index]),
-                         {'action': 'requeue', 'job_ids': job_ids})
-
-        # check if we requeue all failed jobs
-        for job in jobs:
-            self.assertFalse(job.is_failed)
-
-    def test_clear_queue(self):
-        """Test that the queue clear actually clears the queue."""
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-        job = queue.enqueue(access_self)
-        self.client.post(reverse('rq_clear', args=[queue_index]),
-                         {'post': 'yes'})
-        self.assertFalse(Job.exists(job.id, connection=queue.connection))
-        self.assertNotIn(job.id, queue.job_ids)
-
-    def test_finished_jobs(self):
-        """Ensure that finished jobs page works properly."""
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-
-        job = queue.enqueue(access_self)
-        registry = FinishedJobRegistry(queue.name, queue.connection)
-        registry.add(job, 2)
-        response = self.client.get(
-            reverse('rq_finished_jobs', args=[queue_index])
-        )
-        self.assertEqual(response.context['jobs'], [job])
-
-    def test_started_jobs(self):
-        """Ensure that active jobs page works properly."""
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-
-        job = queue.enqueue(access_self)
-        registry = StartedJobRegistry(queue.name, queue.connection)
-        registry.add(job, 2)
-        response = self.client.get(
-            reverse('rq_started_jobs', args=[queue_index])
-        )
-        self.assertEqual(response.context['jobs'], [job])
-
-    def test_deferred_jobs(self):
-        """Ensure that active jobs page works properly."""
-        queue = get_queue('django_rq_test')
-        queue_index = get_queue_index('django_rq_test')
-
-        job = queue.enqueue(access_self)
-        registry = DeferredJobRegistry(queue.name, queue.connection)
-        registry.add(job, 2)
-        response = self.client.get(
-            reverse('rq_deferred_jobs', args=[queue_index])
-        )
-        self.assertEqual(response.context['jobs'], [job])
+    @patch('django_rq.management.commands.rqworker.setup_loghandlers')
+    def test_commandline_verbosity_affects_logging_level(self, setup_loghandlers_mock):
+        expected_level = {
+            0: 'WARNING',
+            1: 'INFO',
+            2: 'DEBUG',
+            3: 'DEBUG',
+        }
+        for verbosity in [0, 1, 2, 3]:
+            setup_loghandlers_mock.reset_mock()
+            call_command('rqworker', verbosity=verbosity, burst=True)
+            setup_loghandlers_mock.assert_called_once_with(expected_level[verbosity])
 
 
 class ThreadQueueTest(TestCase):
@@ -596,28 +585,69 @@ class SchedulerTest(TestCase):
         self.assertEqual(connection_kwargs['port'], config['PORT'])
         self.assertEqual(connection_kwargs['db'], config['DB'])
 
+    @skipIf(RQ_SCHEDULER_INSTALLED is False, 'RQ Scheduler not installed')
+    @patch('django_rq.management.commands.rqscheduler.get_scheduler')
+    @patch('django_rq.management.commands.rqscheduler.setup_loghandlers')
+    def test_commandline_verbosity_affects_logging_level(self, setup_loghandlers_mock, get_scheduler_mock):
+        get_scheduler_mock.run.return_value = None
+        expected_level = {
+            0: 'WARNING',
+            1: 'INFO',
+            2: 'DEBUG',
+            3: 'DEBUG',
+        }
+        for verbosity in [0, 1, 2, 3]:
+            setup_loghandlers_mock.reset_mock()
+            call_command('rqscheduler', verbosity=verbosity)
+            setup_loghandlers_mock.assert_called_once_with(expected_level[verbosity])
+
+    @override_settings(RQ={'SCHEDULER_CLASS': 'django_rq.tests.fixtures.DummyScheduler'})
+    @skipIf(RQ_SCHEDULER_INSTALLED is False, 'RQ Scheduler not installed')
+    def test_scheduler_default_timeout(self):
+        """
+        Scheduler class customization.
+        """
+        scheduler = get_scheduler('default')
+        self.assertIsInstance(scheduler, DummyScheduler)
+
+    @override_settings(RQ={'AUTOCOMMIT': True})
+    @skipIf(RQ_SCHEDULER_INSTALLED is False, 'RQ Scheduler not installed')
+    def test_scheduler_default_timeout(self):
+        """
+        Ensure scheduler respects DEFAULT_RESULT_TTL value for `result_ttl` param.
+        """
+        scheduler = get_scheduler('test_scheduler')
+        job = scheduler.enqueue_at(datetime.datetime.now() + datetime.timedelta(days=1), divide, 1, 1)
+        self.assertTrue(job in scheduler.get_jobs())
+        self.assertEqual(job.timeout, 400)
+        job.delete()
+
+    @override_settings(RQ={'AUTOCOMMIT': True, 'DEFAULT_RESULT_TTL': 5432})
+    @skipIf(RQ_SCHEDULER_INSTALLED is False, 'RQ Scheduler not installed')
+    def test_scheduler_default_result_ttl(self):
+        """
+        Ensure scheduler respects DEFAULT_RESULT_TTL value for `result_ttl` param.
+        """
+        scheduler = get_scheduler('test_scheduler')
+        job = scheduler.enqueue_at(datetime.datetime.now() + datetime.timedelta(days=1), divide, 1, 1)
+        self.assertTrue(job in scheduler.get_jobs())
+        self.assertEqual(job.result_ttl, 5432)
+        job.delete()
+
 
 class RedisCacheTest(TestCase):
 
     @skipIf(settings.REDIS_CACHE_TYPE != 'django-redis',
             'django-redis not installed')
-    def test_get_queue_django_redis(self):
+    @patch('django_redis.get_redis_connection')
+    def test_get_queue_django_redis(self, mocked):
         """
         Test that the USE_REDIS_CACHE option for configuration works.
         """
-        queueName = 'django-redis'
-        queue = get_queue(queueName)
-        connection_kwargs = queue.connection.connection_pool.connection_kwargs
-        self.assertEqual(queue.name, queueName)
-
-        cacheHost = settings.CACHES[queueName]['LOCATION'].split(':')[0]
-        cachePort = settings.CACHES[queueName]['LOCATION'].split(':')[1]
-        cacheDBNum = settings.CACHES[queueName]['LOCATION'].split(':')[2]
-
-        self.assertEqual(connection_kwargs['host'], cacheHost)
-        self.assertEqual(connection_kwargs['port'], int(cachePort))
-        self.assertEqual(connection_kwargs['db'], int(cacheDBNum))
-        self.assertEqual(connection_kwargs['password'], None)
+        queue = get_queue('django-redis')
+        queue.enqueue(access_self)
+        self.assertEqual(len(queue), 1)
+        self.assertEqual(mocked.call_count, 1)
 
     @skipIf(settings.REDIS_CACHE_TYPE != 'django-redis-cache',
             'django-redis-cache not installed')
@@ -640,8 +670,22 @@ class RedisCacheTest(TestCase):
         self.assertEqual(connection_kwargs['password'], None)
 
 
-class DummyQueue(DjangoRQ):
-    """Just Fake class for the following test"""
+class JobClassTest(TestCase):
+
+    def test_default_job_class(self):
+        job_class = get_job_class()
+        self.assertIs(job_class, Job)
+
+    @override_settings(RQ={'JOB_CLASS': 'django_rq.tests.fixtures.DummyJob'})
+    def test_custom_class(self):
+        job_class = get_job_class()
+        self.assertIs(job_class, DummyJob)
+
+    def test_local_override(self):
+        self.assertIs(
+            get_job_class('django_rq.tests.fixtures.DummyJob'),
+            DummyJob
+        )
 
 
 class QueueClassTest(TestCase):
@@ -659,6 +703,24 @@ class QueueClassTest(TestCase):
         self.assertIsInstance(queue, DummyQueue)
 
 
+class WorkerClassTest(TestCase):
+
+    def test_default_worker_class(self):
+        worker = get_worker('test')
+        self.assertIsInstance(worker, Worker)
+
+    @override_settings(RQ={'WORKER_CLASS': 'django_rq.tests.fixtures.DummyWorker'})
+    def test_custom_class(self):
+        worker = get_worker('test')
+        self.assertIsInstance(worker, DummyWorker)
+
+    def test_local_override(self):
+        self.assertIs(
+            get_worker_class('django_rq.tests.fixtures.DummyWorker'),
+            DummyWorker
+        )
+
+
 @override_settings(RQ={'AUTOCOMMIT': True})
 class TemplateTagTest(TestCase):
 
@@ -670,3 +732,60 @@ class TemplateTagTest(TestCase):
 
             self.assertIsNotNone(time.tzinfo)
             self.assertEqual(time.strftime("%z"), '+0700')
+
+
+class UtilsTest(TestCase):
+
+    def test_get_statistics(self):
+        """get_statistics() returns the right number of workers"""
+        queues = [{
+            'connection_config': {
+                'DB': 0,
+                'HOST': 'localhost',
+                'PORT': 6379,
+            },
+            'name': 'async'
+        }]
+
+        with patch('django_rq.utils.QUEUES_LIST',
+                   new_callable=PropertyMock(return_value=queues)):
+            worker = get_worker('async', name=uuid4().hex)
+            worker.register_birth()
+            statistics = get_statistics()
+            data = statistics['queues'][0]
+            self.assertEqual(data['name'], 'async')
+            self.assertEqual(data['workers'], 1)
+            worker.register_death()
+
+    def test_get_jobs(self):
+        """get_jobs() works properly"""
+        queue = get_queue('django_rq_test')
+
+        registry = ScheduledJobRegistry(queue.name, queue.connection)
+        flush_registry(registry)
+
+        now = datetime.datetime.now()
+        job = queue.enqueue_at(now, access_self)
+        job2 = queue.enqueue_at(now, access_self)
+        self.assertEqual(
+            get_jobs(queue, [job.id, job2.id]),
+            [job, job2]
+        )
+        self.assertEqual(len(registry), 2)
+
+        # job has been deleted, so the result will be filtered out
+        queue.connection.delete(job.key)
+        self.assertEqual(
+            get_jobs(queue, [job.id, job2.id]),
+            [job2]
+        )
+        self.assertEqual(len(registry), 2)
+
+        # If job has been deleted and `registry` is passed,
+        # job will also be removed from registry
+        queue.connection.delete(job2.key)
+        self.assertEqual(
+            get_jobs(queue, [job.id, job2.id], registry),
+            []
+        )
+        self.assertEqual(len(registry), 0)
\ No newline at end of file
