@@ -2,62 +2,48 @@ from __future__ import division
 
 from math import ceil
 
-from django.contrib import messages
+from django.contrib import admin, messages
 from django.contrib.admin.views.decorators import staff_member_required
-from django.http import Http404
+from django.http import Http404, JsonResponse
 from django.shortcuts import redirect, render
 
 from redis.exceptions import ResponseError
-from rq import requeue_job, Worker
-from rq.exceptions import NoSuchJobError
-from rq.job import Job
-from rq.registry import (DeferredJobRegistry, FinishedJobRegistry,
-                         StartedJobRegistry)
+from rq import requeue_job
+from rq.exceptions import NoSuchJobError, UnpickleError
+from rq.job import Job, JobStatus
+from rq.registry import (
+    DeferredJobRegistry, 
+    FailedJobRegistry, 
+    FinishedJobRegistry, 
+    ScheduledJobRegistry,
+    StartedJobRegistry, 
+)
+from rq.worker import Worker
 
-from .queues import get_connection, get_queue_by_index
-from .settings import QUEUES_LIST
+from .queues import get_queue_by_index
+from .settings import API_TOKEN
+from .utils import get_statistics, get_jobs
 
 
 @staff_member_required
 def stats(request):
-    queues = []
-    for index, config in enumerate(QUEUES_LIST):
-
-        queue = get_queue_by_index(index)
-        connection = queue.connection
-
-        queue_data = {
-            'name': queue.name,
-            'jobs': queue.count,
-            'index': index,
-            'connection_kwargs': connection.connection_pool.connection_kwargs
-        }
-
-        if queue.name == 'failed':
-            queue_data['workers'] = '-'
-            queue_data['finished_jobs'] = '-'
-            queue_data['started_jobs'] = '-'
-            queue_data['deferred_jobs'] = '-'
-
-        else:
-            connection = get_connection(queue.name)
-            all_workers = Worker.all(connection=connection)
-            queue_workers = [worker for worker in all_workers if queue in worker.queues]
-            queue_data['workers'] = len(queue_workers)
-
-            finished_job_registry = FinishedJobRegistry(queue.name, connection)
-            started_job_registry = StartedJobRegistry(queue.name, connection)
-            deferred_job_registry = DeferredJobRegistry(queue.name, connection)
-            queue_data['finished_jobs'] = len(finished_job_registry)
-            queue_data['started_jobs'] = len(started_job_registry)
-            queue_data['deferred_jobs'] = len(deferred_job_registry)
-
-        queues.append(queue_data)
-
-    context_data = {'queues': queues}
+    context_data = {
+        **admin.site.each_context(request),
+        **get_statistics(run_maintenance_tasks=True)
+    }
     return render(request, 'django_rq/stats.html', context_data)
 
 
+def stats_json(request, token=None):
+    if request.user.is_staff or (token and token == API_TOKEN):
+        return JsonResponse(get_statistics())
+
+    return JsonResponse({
+        "error": True,
+        "description": "Please configure API_TOKEN in settings.py before accessing this view."
+    })
+
+
 @staff_member_required
 def jobs(request, queue_index):
     queue_index = int(queue_index)
@@ -77,6 +63,7 @@ def jobs(request, queue_index):
         page_range = []
 
     context_data = {
+        **admin.site.each_context(request),
         'queue': queue,
         'queue_index': queue_index,
         'jobs': jobs,
@@ -105,17 +92,13 @@ def finished_jobs(request, queue_index):
         page_range = range(1, last_page + 1)
         offset = items_per_page * (page - 1)
         job_ids = registry.get_job_ids(offset, offset + items_per_page - 1)
-
-        for job_id in job_ids:
-            try:
-                jobs.append(Job.fetch(job_id, connection=queue.connection))
-            except NoSuchJobError:
-                pass
+        jobs = get_jobs(queue, job_ids, registry)
 
     else:
         page_range = []
 
     context_data = {
+        **admin.site.each_context(request),
         'queue': queue,
         'queue_index': queue_index,
         'jobs': jobs,
@@ -127,6 +110,79 @@ def finished_jobs(request, queue_index):
     return render(request, 'django_rq/jobs.html', context_data)
 
 
+@staff_member_required
+def failed_jobs(request, queue_index):
+    queue_index = int(queue_index)
+    queue = get_queue_by_index(queue_index)
+
+    registry = FailedJobRegistry(queue.name, queue.connection)
+
+    items_per_page = 100
+    num_jobs = len(registry)
+    page = int(request.GET.get('page', 1))
+    jobs = []
+
+    if num_jobs > 0:
+        last_page = int(ceil(num_jobs / items_per_page))
+        page_range = range(1, last_page + 1)
+        offset = items_per_page * (page - 1)
+        job_ids = registry.get_job_ids(offset, offset + items_per_page - 1)
+        jobs = get_jobs(queue, job_ids, registry)
+
+    else:
+        page_range = []
+
+    context_data = {
+        **admin.site.each_context(request),
+        'queue': queue,
+        'queue_index': queue_index,
+        'jobs': jobs,
+        'num_jobs': num_jobs,
+        'page': page,
+        'page_range': page_range,
+        'job_status': 'Failed',
+    }
+    return render(request, 'django_rq/jobs.html', context_data)
+
+
+@staff_member_required
+def scheduled_jobs(request, queue_index):
+    queue_index = int(queue_index)
+    queue = get_queue_by_index(queue_index)
+
+    registry = ScheduledJobRegistry(queue.name, queue.connection)
+
+    items_per_page = 100
+    num_jobs = len(registry)
+    page = int(request.GET.get('page', 1))
+    jobs = []
+
+    if num_jobs > 0:
+        last_page = int(ceil(num_jobs / items_per_page))
+        page_range = range(1, last_page + 1)
+        offset = items_per_page * (page - 1)
+        job_ids = registry.get_job_ids(offset, offset + items_per_page - 1)
+
+        jobs = get_jobs(queue, job_ids, registry)
+        for job in jobs:
+            job.scheduled_at = registry.get_scheduled_time(job)
+
+    else:
+        page_range = []
+
+    context_data = {
+        **admin.site.each_context(request),
+        'queue': queue,
+        'queue_index': queue_index,
+        'jobs': jobs,
+        'num_jobs': num_jobs,
+        'page': page,
+        'page_range': page_range,
+        'job_status': 'Scheduled',
+    }
+    return render(request, 'django_rq/jobs.html', context_data)
+
+
 @staff_member_required
 def started_jobs(request, queue_index):
     queue_index = int(queue_index)
@@ -144,17 +200,13 @@ def started_jobs(request, queue_index):
         page_range = range(1, last_page + 1)
         offset = items_per_page * (page - 1)
         job_ids = registry.get_job_ids(offset, offset + items_per_page - 1)
-
-        for job_id in job_ids:
-            try:
-                jobs.append(Job.fetch(job_id, connection=queue.connection))
-            except NoSuchJobError:
-                pass
+        jobs = get_jobs(queue, job_ids, registry)
 
     else:
         page_range = []
 
     context_data = {
+        **admin.site.each_context(request),
         'queue': queue,
         'queue_index': queue_index,
         'jobs': jobs,
@@ -166,6 +218,45 @@ def started_jobs(request, queue_index):
     return render(request, 'django_rq/jobs.html', context_data)
 
 
+@staff_member_required
+def workers(request, queue_index):
+    queue_index = int(queue_index)
+    queue = get_queue_by_index(queue_index)
+    all_workers = Worker.all(queue.connection)
+    workers = [worker for worker in all_workers
+               if queue.name in worker.queue_names()]
+
+    context_data = {
+        **admin.site.each_context(request),
+        'queue': queue,
+        'queue_index': queue_index,
+        'workers': workers,
+    }
+    return render(request, 'django_rq/workers.html', context_data)
+
+
+@staff_member_required
+def worker_details(request, queue_index, key):
+    queue_index = int(queue_index)
+    queue = get_queue_by_index(queue_index)
+    worker = Worker.find_by_key(key, connection=queue.connection)
+    # Convert microseconds to milliseconds
+    worker.total_working_time = worker.total_working_time / 1000
+
+    queue_names = ', '.join(worker.queue_names())
+
+    context_data = {
+        **admin.site.each_context(request),
+        'queue': queue,
+        'queue_index': queue_index,
+        'worker': worker,
+        'queue_names': queue_names,
+        'job': worker.get_current_job(),
+        'total_working_time': worker.total_working_time * 1000
+    }
+    return render(request, 'django_rq/worker_details.html', context_data)
+
+
 @staff_member_required
 def deferred_jobs(request, queue_index):
     queue_index = int(queue_index)
@@ -194,6 +285,7 @@ def deferred_jobs(request, queue_index):
         page_range = []
 
     context_data = {
+        **admin.site.each_context(request),
         'queue': queue,
         'queue_index': queue_index,
         'jobs': jobs,
@@ -214,10 +306,19 @@ def job_detail(request, queue_index, job_id):
     except NoSuchJobError:
         raise Http404("Couldn't find job with this ID: %s" % job_id)
 
+    try:
+        job.func_name
+        data_is_valid = True
+    except UnpickleError:
+        data_is_valid = False
+
     context_data = {
+        **admin.site.each_context(request),
         'queue_index': queue_index,
         'job': job,
+        'dependency_id': job._dependency_id, 
         'queue': queue,
+        'data_is_valid': data_is_valid
     }
     return render(request, 'django_rq/job_detail.html', context_data)
 
@@ -230,12 +331,13 @@ def delete_job(request, queue_index, job_id):
 
     if request.method == 'POST':
         # Remove job id from queue and delete the actual job
-        queue.connection._lrem(queue.key, 0, job.id)
+        queue.connection.lrem(queue.key, 0, job.id)
         job.delete()
         messages.info(request, 'You have successfully deleted %s' % job.id)
         return redirect('rq_jobs', queue_index)
 
     context_data = {
+        **admin.site.each_context(request),
         'queue_index': queue_index,
         'job': job,
         'queue': queue,
@@ -255,6 +357,7 @@ def requeue_job_view(request, queue_index, job_id):
         return redirect('rq_job_detail', queue_index, job_id)
 
     context_data = {
+        **admin.site.each_context(request),
         'queue_index': queue_index,
         'job': job,
         'queue': queue,
@@ -279,6 +382,7 @@ def clear_queue(request, queue_index):
         return redirect('rq_jobs', queue_index)
 
     context_data = {
+        **admin.site.each_context(request),
         'queue_index': queue_index,
         'queue': queue,
     }
@@ -289,20 +393,27 @@ def clear_queue(request, queue_index):
 def requeue_all(request, queue_index):
     queue_index = int(queue_index)
     queue = get_queue_by_index(queue_index)
-    jobs = queue.get_jobs()
+    registry = FailedJobRegistry(queue=queue)
 
     if request.method == 'POST':
+        job_ids = registry.get_job_ids()
+        count = 0
         # Confirmation received
-        for job in jobs:
-            requeue_job(job.id, connection=queue.connection)
+        for job_id in job_ids:
+            try:
+                requeue_job(job_id, connection=queue.connection)
+                count += 1
+            except NoSuchJobError:
+                pass
 
-        messages.info(request, 'You have successfully requeued all %d jobs!' % len(jobs))
+        messages.info(request, 'You have successfully requeued %d jobs!' % count)
         return redirect('rq_jobs', queue_index)
 
     context_data = {
+        **admin.site.each_context(request),
         'queue_index': queue_index,
         'queue': queue,
-        'total_jobs':len(jobs),
+        'total_jobs': len(registry),
     }
 
     return render(request, 'django_rq/requeue_all.html', context_data)
@@ -317,6 +428,7 @@ def actions(request, queue_index):
         # confirm action
         if request.POST.get('_selected_action', False):
             context_data = {
+                **admin.site.each_context(request),
                 'queue_index': queue_index,
                 'action': request.POST['action'],
                 'job_ids': request.POST.getlist('_selected_action'),
@@ -332,7 +444,7 @@ def actions(request, queue_index):
                 for job_id in job_ids:
                     job = Job.fetch(job_id, connection=queue.connection)
                     # Remove job id from queue and delete the actual job
-                    queue.connection._lrem(queue.key, 0, job.id)
+                    queue.connection.lrem(queue.key, 0, job.id)
                     job.delete()
                 messages.info(request, 'You have successfully deleted %s jobs!' % len(job_ids))
             elif request.POST['action'] == 'requeue':
@@ -341,3 +453,34 @@ def actions(request, queue_index):
                 messages.info(request, 'You have successfully requeued %d  jobs!' % len(job_ids))
 
     return redirect('rq_jobs', queue_index)
+
+
+@staff_member_required
+def enqueue_job(request, queue_index, job_id):
+    """ Enqueue deferred jobs
+    """
+    queue_index = int(queue_index)
+    queue = get_queue_by_index(queue_index)
+    job = Job.fetch(job_id, connection=queue.connection)
+
+    if request.method == 'POST':
+        queue.enqueue_job(job)
+
+        # Remove job from correct registry if needed
+        if job.get_status() == JobStatus.DEFERRED:
+            registry = DeferredJobRegistry(queue.name, queue.connection)
+            registry.remove(job)
+        elif job.get_status() == JobStatus.FINISHED:
+            registry = FinishedJobRegistry(queue.name, queue.connection)
+            registry.remove(job)
+
+        messages.info(request, 'You have successfully enqueued %s' % job.id)
+        return redirect('rq_job_detail', queue_index, job_id)
+
+    context_data = {
+        **admin.site.each_context(request),
+        'queue_index': queue_index,
+        'job': job,
+        'queue': queue,
+    }
+    return render(request, 'django_rq/delete_job.html', context_data)
